@@ -24,8 +24,7 @@ module Control.Monad.Writer.Class (
     censor,
   ) where
 
-import Control.Monad
-import Control.Monad.Trans.Error as Error
+import Control.Monad.Trans.Except as Except
 import Control.Monad.Trans.Identity as Identity
 import Control.Monad.Trans.Maybe as Maybe
 import Control.Monad.Trans.Reader
@@ -40,31 +39,41 @@ import qualified Control.Monad.Trans.Writer.Lazy as Lazy (
 import qualified Control.Monad.Trans.Writer.Strict as Strict (
         WriterT, tell, listen, pass)
 import Control.Monad.Trans (lift)
-import Data.Monoid
 
 -- ---------------------------------------------------------------------------
 -- MonadWriter class
---
--- tell is like tell on the MUD's it shouts to monad
--- what you want to be heard. The monad carries this 'packet'
--- upwards, merging it if needed (hence the Monoid requirement).
---
--- listen listens to a monad acting, and returns what the monad "said".
---
--- pass lets you provide a writer transformer which changes internals of
--- the written object.
 
 class (Monoid (WriterType m), Monad m) => MonadWriter m where
     type WriterType m
+
+    -- | Shout to the monad what you want to be heard. The monad carries
+    -- this packet upwards, merging it if needed (hence the 'Monoid'
+    -- requirement).
     tell   :: WriterType m -> m ()
+
+    -- | Listen to a monad acting, and return what the monad "said".
     listen :: m a -> m (a, WriterType m)
+
+    -- | Provide a writer transformer which changes internals of the
+    -- written object.
     pass   :: m (a, WriterType m -> WriterType m) -> m a
 
+-- | @'listens' f m@ is an action that executes the action @m@ and adds
+-- the result of applying @f@ to the output to the value of the computation.
+--
+-- * @'listens' f m = 'liftM' (id *** f) ('listen' m)@
+--
 listens :: (MonadWriter m) => (WriterType m -> b) -> m a -> m (a, b)
 listens f m = do
     ~(a, w) <- listen m
     return (a, f w)
 
+-- | @'censor' f m@ is an action that executes the action @m@ and
+-- applies the function @f@ to its output, leaving the return value
+-- unchanged.
+--
+-- * @'censor' f m = 'pass' ('liftM' (\\ x -> (x,f)) m)@
+--
 censor :: (MonadWriter m) => (WriterType m -> WriterType m) -> m a -> m a
 censor f m = pass $ do
     a <- m
@@ -97,11 +106,11 @@ instance (Monoid w, Monad m) => MonadWriter (StrictRWS.RWST r w s m) where
 -- ---------------------------------------------------------------------------
 -- Instances for other mtl transformers
 
-instance (Error e, MonadWriter m) => MonadWriter (ErrorT e m) where
-    type WriterType (ErrorT e m) = WriterType m
+instance (MonadWriter m) => MonadWriter (ExceptT e m) where
+    type WriterType (ExceptT e m) = WriterType m
     tell   = lift . tell
-    listen = Error.liftListen listen
-    pass   = Error.liftPass pass
+    listen = Except.liftListen listen
+    pass   = Except.liftPass pass
 
 instance (MonadWriter m) => MonadWriter (IdentityT m) where
     type WriterType (IdentityT m) = WriterType m
